"""
  Menu Editor - A simple menu editor

  Travis Watkins <alleykat@gmail.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  (C) Copyright 2005 Travis Watkins
"""

import xml.dom.minidom
import string, os, string, locale
import xdg.Menu, xdg.DesktopEntry

class MenuHandler:
    cat_list = []
    menus = []

    def __init__(self):
        home_dir = os.path.expanduser('~')
        config_dir = os.path.join(home_dir, '.config/menus/')
        if not os.access(os.path.join(home_dir, '.config/menus/'), os.F_OK):
            os.makedirs(config_dir)
        self.menu = os.path.join(config_dir, 'applications.menu')
        if not os.access(self.menu, os.F_OK):
            open(self.menu, 'w').write("""
<!DOCTYPE Menu PUBLIC "-//freedesktop//DTD Menu 1.0//EN"
"http://www.freedesktop.org/standards/menu-spec/1.0/menu.dtd">
<Menu>
  <Name>Applications</Name>
  <MergeFile>/etc/xdg/menus/applications.menu</MergeFile>
</Menu>
            """)
        self.domtree = xml.dom.minidom.parse(self.menu)

    def intToCat(self, integer):
        if integer > (len(self.cat_list) - 1):
            return None
        return self.cat_list[integer]

    def catToInt(self, cat):
        i = 0
        while i < len(self.cat_list):
            if cat == self.cat_list[i]:
                return i
            i += 1
        return None

    def addCategoryMenu(self, category):
        menu = xml.dom.minidom.Element('Menu')
        name = xml.dom.minidom.Element('Name')
        text = xml.dom.minidom.Text()
        text.data = category
        name.appendChild(text)
        menu.appendChild(name)
        self.domtree.childNodes[1].appendChild(menu)
        return self.getCategoryMenu(category)

    def getCategoryMenu(self, category):
        for menu in self.domtree.childNodes[1].getElementsByTagName('Menu'):
            for element in menu.childNodes:
                if element.nodeName != '#text':
                    try:
                        if element.childNodes[0].data == category:
                            return menu
                    except:
                        pass
        return self.addCategoryMenu(category)

    def addUserOverride(self, rootname, username, category):
        menu = self.getCategoryMenu(category)
        menu.ownerDocument = self.domtree
        if username:
            include = xml.dom.minidom.Element('Include')
            include_name = xml.dom.minidom.Element('Filename')
            include_text = xml.dom.minidom.Text()
            include_text.data = username
            include_name.appendChild(include_text)
            include.appendChild(include_name)
            menu.appendChild(include)
        exclude = xml.dom.minidom.Element('Exclude')
        exclude_name = xml.dom.minidom.Element('Filename')
        exclude_text = xml.dom.minidom.Text()
        exclude_text.data = rootname
        exclude_name.appendChild(exclude_text)
        exclude.appendChild(exclude_name)
        menu.appendChild(exclude)
        self.writeMenuFile()

    def writeMenuFile(self):
        open(self.menu, 'w').write(self.domtree.toxml().replace('<?xml version="1.0" ?>\n', ''))

    def loadMenus(self, parent=None):
        if not parent:
            menu = xdg.Menu.parse(self.menu)
            menu.setWM('GNOME')
            try:
                self.locale = locale.getdefaultlocale()[0]
                menu.setLocale(self.locale)
            except:
                self.locale = ''
            self.menus = []
        else:
            menu = parent

        for entry in menu.getEntries():
            if isinstance(entry, xdg.Menu.Menu):
                self.cat_list.append(entry.getName())
                self.menus.append([entry, []])
                self.loadMenus(entry)
            elif isinstance(entry, xdg.Menu.MenuEntry):
                entry.DesktopEntry.display_category = self.menus[-1][0].getName()
                if entry.DesktopEntry.content.has_key('Desktop Entry') and entry.DesktopEntry.content['Desktop Entry'].has_key('OnlyShowIn'):
                    if 'GNOME' in entry.DesktopEntry.content['Desktop Entry']['OnlyShowIn']:
                        self.menus[-1][1].append(entry.DesktopEntry)
                else:
                    self.menus[-1][1].append(entry.DesktopEntry)

    def saveEntry(self, entry, name, comment, command, icon, useterm, category, filename):
        if entry:
            newentry = entry
        else:
            newentry = xdg.DesktopEntry.DesktopEntry()
            newentry.addGroup('Desktop Entry')
            newentry.set('Encoding', 'UTF-8', 'Desktop Entry')
            newentry.set('Type', 'Application', 'Desktop Entry')
        if newentry.content['Desktop Entry'].has_key('Name[' + self.locale + ']'):
            newentry.content['Desktop Entry']['Name[' + self.locale + ']'] = name.encode('utf-8', 'ignore')
        newentry.set('Name', name, 'Desktop Entry')
        if newentry.content['Desktop Entry'].has_key('Comment[' + self.locale + ']'):
            newentry.content['Desktop Entry']['Comment[' + self.locale + ']'] = comment.encode('utf-8', 'ignore')
        newentry.set('Comment', comment, 'Desktop Entry')
        newentry.set('Exec', command, 'Desktop Entry')
        if useterm == True:
            newentry.set('Terminal', 'true', 'Desktop Entry')
        else:
            newentry.set('Terminal', 'false', 'Desktop Entry')
        newentry.set('Categories', 'Application;' + category + ';', 'Desktop Entry')
        try:
            newentry.set('Icon', icon.my_icon_name, 'Desktop Entry')
        except:
            pass
        newentry.write(filename)
